/* Copyright c 2005-2012.
 * Licensed under GNU  LESSER General Public License, Version 3.
 * http://www.gnu.org/licenses
 */
package org.beangle.security.blueprint.service.internal;

import java.sql.Date;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.beangle.commons.collection.CollectUtils;
import org.beangle.commons.dao.impl.BaseServiceImpl;
import org.beangle.commons.dao.query.builder.OqlBuilder;
import org.beangle.commons.lang.Objects;
import org.beangle.commons.lang.Strings;
import org.beangle.security.auth.Principals;
import org.beangle.security.blueprint.Member;
import org.beangle.security.blueprint.Role;
import org.beangle.security.blueprint.User;
import org.beangle.security.blueprint.event.UserAlterationEvent;
import org.beangle.security.blueprint.event.UserCreationEvent;
import org.beangle.security.blueprint.event.UserRemoveEvent;
import org.beangle.security.blueprint.event.UserStatusEvent;
import org.beangle.security.blueprint.model.MemberBean;
import org.beangle.security.blueprint.service.UserService;

/**
 * 用户信息服务的实现类
 * 
 * @author dell,chaostone 2005-9-26
 */
public class UserServiceImpl extends BaseServiceImpl implements UserService {

  public boolean isRoot(User user) {
    return Principals.ROOT.equals(user.getId());
  }

  public boolean isRoot(Long userId) {
    return Principals.ROOT.equals(userId);
  }

  public User get(String name, String password) {
    Map<String, Object> params = CollectUtils.newHashMap();
    params.put("name", name);
    params.put("password", password);
    List<?> userList = entityDao.searchHQLQuery(
        "from User user where  user.name = :name and user.password = :password", params);
    if (userList.size() > 0) return (User) userList.get(0);
    else return null;
  }

  public User get(Long id) {
    return entityDao.get(User.class, id);
  }

  public User get(String loginName) {
    if (Strings.isEmpty(loginName)) return null;
    OqlBuilder<User> entityQuery = OqlBuilder.from(User.class, "user");
    entityQuery.where("user.name=:name", loginName);
    List<User> users = entityDao.search(entityQuery);
    return (users.isEmpty()) ? null : users.get(0);
  }

  public List<User> getUsers(Long[] userIds) {
    return entityDao.get(User.class, userIds);
  }

  public int updateState(final User manager, Long[] ids, final boolean enabled) {
    assert (null == ids || ids.length < 1);
    @SuppressWarnings("unchecked")
    List<User> users = (List<User>) CollectionUtils.select(getUsers(ids), new Predicate() {
      public boolean evaluate(Object object) {
        User one = (User) object;
        return isManagedBy(manager, one) && !manager.equals(one) && (one.isEnabled() != enabled);
      }
    });

    for (int i = 0; i < users.size(); i++) {
      User cur = users.get(i);
      cur.setEnabled(enabled);
    }
    if (!users.isEmpty()) {
      entityDao.saveOrUpdate(users);
      publish(new UserStatusEvent(users, enabled));
    }
    return users.size();
  }

  public void saveOrUpdate(User user) {
    user.setUpdatedAt(new Date(System.currentTimeMillis()));
    if (!user.isPersisted()) user.setCreatedAt(new Date(System.currentTimeMillis()));
    entityDao.saveOrUpdate(user);
    publish(new UserAlterationEvent(Collections.singletonList(user)));
  }

  // workground for no session
  @SuppressWarnings({ "rawtypes", "unchecked" })
  public List<Role> getRoles(Long userId) {
    OqlBuilder builder = OqlBuilder.from(Member.class, "gm");
    builder.where("gm.user.id=:userId and gm.member=true", userId).select("gm.role")
        .orderBy("gm.role.code");
    builder.cacheable();
    return entityDao.search(builder);
  }

  public List<Member> getMembers(User user, Member.Ship ship) {
    if (isRoot(user) && !Objects.equals(ship, Member.Ship.MEMBER)) {
      List<Member> members = CollectUtils.newArrayList();
      List<Role> roles = entityDao.getAll(Role.class);
      for (Role role : roles) {
        MemberBean gmb = new MemberBean(role, user, Member.Ship.MEMBER);
        gmb.setGranter(true);
        gmb.setManager(true);
        members.add(gmb);
      }
      return members;
    }
    OqlBuilder<Member> builder = OqlBuilder.from(Member.class, "gm");
    builder.where("gm.user=:user", user);
    if (null != ship) {
      if (ship.equals(Member.Ship.MEMBER)) builder.where("gm.member=true");
      if (ship.equals(Member.Ship.MANAGER)) builder.where("gm.manager=true");
      if (ship.equals(Member.Ship.GRANTER)) builder.where("gm.granter=true");
    }
    return entityDao.search(builder);
  }

  public void createUser(User creator, User newUser) {
    newUser.setCreator(creator);
    newUser.setUpdatedAt(new Date(System.currentTimeMillis()));
    newUser.setCreatedAt(new Date(System.currentTimeMillis()));
    entityDao.saveOrUpdate(newUser);
    publish(new UserCreationEvent(Collections.singletonList(newUser)));
  }

  public void removeUser(User manager, User user) {
    List<User> removed = CollectUtils.newArrayList();
    if (isManagedBy(manager, user)) {
      entityDao.remove(user);
      removed.add(user);
    }
    publish(new UserRemoveEvent(removed));
  }

  public boolean isManagedBy(User manager, User user) {
    return (isRoot(manager) || manager.equals(user.getCreator()));
  }

}
